package org.erif.util;

import java.sql.*;
import java.text.*;
import java.util.*;
import javax.servlet.http.*;

/** handy class with simple defaults for date formats and some html niftiness */
public class DateUtil {

  public static final String POSTGRESQL_TIMESTAMP = "yyyy-MM-dd HH:mm:ss";
  public static final String POSTGRESQL_DATE = "yyyy-MM-dd";
  public static final int UNKNOWN = -1;
  public static final String shortDate = "d-MMM-yy/h:mm a";
  public static final String longDate = "MMMM d, yyyy h:mm a zzz";
  public static final String shrunkDate = "dd-MMM-yy";
  public static final String shrunkTime = "hh:mm a";

  /** Returns a String array translating common names for months (displayed in date pulldown) to java.util.Calendar constants   */
  public static final String[][] months = {
    {"Jan", String.valueOf(Calendar.JANUARY)},
    {"Feb", String.valueOf(Calendar.FEBRUARY)},
    {"Mar", String.valueOf(Calendar.MARCH)},
    {"Apr", String.valueOf(Calendar.APRIL)},
    {"May", String.valueOf(Calendar.MAY)},
    {"Jun", String.valueOf(Calendar.JUNE)},
    {"Jul", String.valueOf(Calendar.JULY)},
    {"Aug", String.valueOf(Calendar.AUGUST)},
    {"Sep", String.valueOf(Calendar.SEPTEMBER)},
    {"Oct", String.valueOf(Calendar.OCTOBER)},
    {"Nov", String.valueOf(Calendar.NOVEMBER)},
    {"Dec", String.valueOf(Calendar.DECEMBER)}
 };

  /** Returns an int array of days of the month.   */
  public static final int[] days = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
    16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31};

  /** Returns an int array of years   */
  public static final int[] years = {2001,2002,2003,2004,2005,2006,2007,2008,
    2009,2010,2011,2012,2013,2014,2015};

  /** Returns an int array of hours (12h)   */
  public static final int[] hours = {1,2,3,4,5,6,7,8,9,10,11,12};

  /** Returns a string array of minutes (00/15/30/45)   */
  public static final String[] minutes = {"00","15","30","45"};

  /** Returns common name for AM and PM, plus java.util.Calendar constants for each   */
  public static final String[][] ampm = {
    {"AM", String.valueOf(Calendar.AM)},
    {"PM", String.valueOf(Calendar.PM)}
  };

  /** Stores the date and time for this object */
  private java.util.Date date;

  public DateUtil() {
    date = new java.util.Date();
  }

  public DateUtil(java.util.Date date) {
    this.date = date;
  }

  public void setDate(java.util.Date date) {
    this.date = date;
  }

  public java.util.Date getDate() {
    return date;
  }

  /**
   * Creates an HTML form element for inputting a date's month
   *
   * @param def Default date to display
   * @return HTML for date input
   */
  public static String getMonthInputForm(int i) {
    String month = HTMLUtil.makeSelectList("month", DateUtil.months,
      String.valueOf(i));
    return month;
  }

  public static String getMonthInputForm(java.util.Date def) {
    Calendar cal = Calendar.getInstance();
    if (def != null) cal.setTime(def);
    return getMonthInputForm(cal.get(Calendar.MONTH));
  }

  public String getMonthInputForm() {
    return getInputForm(this.date);
  }


  /**
   * Creates an HTML form element for inputting a date
   *
   * @param def Default date to display
   * @return HTML for date input
   */
  public static String getInputForm(java.util.Date def) {
    Calendar cal = Calendar.getInstance();
    if (def != null)
      cal.setTime(def);

    int min = cal.get(Calendar.MINUTE);
    String mm = "";
    if (min >= 45)
      mm = "45";
    else if (min >= 30)
      mm = "30";
    else if (min >= 15)
      mm = "15";
    else
      mm = "00";

    String month = HTMLUtil.makeSelectList("_month", DateUtil.months,
      String.valueOf(cal.get(Calendar.MONTH)));
    String day = HTMLUtil.makeSelectList("_day", DateUtil.days,
      cal.get(Calendar.DAY_OF_MONTH));
    String year = HTMLUtil.makeSelectList("_year", DateUtil.years,
      cal.get(Calendar.YEAR));
    String hour = HTMLUtil.makeSelectList("_hour", DateUtil.hours,
      cal.get(Calendar.HOUR));
    String minute = HTMLUtil.makeSelectList("_minute", DateUtil.minutes, mm);
    String ampm = HTMLUtil.makeSelectList("_ampm", DateUtil.ampm,
      String.valueOf(cal.get(Calendar.AM_PM)));

    String retval = "<input type=\"radio\" name=\"_useDate\" value=\"n\" ";
    if (def == null)
      retval +=" CHECKED";
    retval +=">No date<br><input type=\"radio\" name=\"_useDate\" value=\"y\" ";
    if (def != null)
      retval +=" CHECKED";
    retval +=">Use date: ";

    return retval+month+"&nbsp;"+day+", &nbsp;"+year+"&nbsp;&nbsp;&nbsp;&nbsp;"
      +hour+":"+minute+"&nbsp;"+ampm;
  }

  public String getInputForm() {
    return getInputForm(this.date);
  }

  public static java.util.Date parseInputForm(HttpServletRequest request) {
    if (request.getParameter("_useDate").equals("n"))
      return null;

    int month = Integer.parseInt(request.getParameter("_month"));
    int day = Integer.parseInt(request.getParameter("_day"));
    int year = Integer.parseInt(request.getParameter("_year"));
    int hour = Integer.parseInt(request.getParameter("_hour"));
    int minute = Integer.parseInt(request.getParameter("_minute"));
    int ampm = Integer.parseInt(request.getParameter("_ampm"));

    Calendar cal = Calendar.getInstance();
    cal.setLenient(false);
    cal.set(Calendar.MONTH, month);
    cal.set(Calendar.DAY_OF_MONTH, day);
    cal.set(Calendar.YEAR, year);
    cal.set(Calendar.HOUR, hour);
    cal.set(Calendar.MINUTE, minute);
    cal.set(Calendar.AM_PM, ampm);
    return cal.getTime();
  }

  public static String displayShrunk(java.util.Date date, String nv) {
    if (date == null) return nv; return displayShrunk(date);
  }

  public static String displayShrunk(java.util.Date date) {
    if (date == null) return null;
    Calendar cal = Calendar.getInstance();
    cal.setTime(new java.util.Date());
    cal.add(Calendar.DAY_OF_MONTH,-1);
    String shrunkFormat;
    if (cal.getTime().before(date)) {
      shrunkFormat = shrunkTime;
    } else {
      shrunkFormat = shrunkDate;
    }
    SimpleDateFormat df = new SimpleDateFormat(shrunkFormat);
    df.setTimeZone(TimeZone.getDefault());
    return df.format(date);
  }

  public static String displayShort(java.util.Date date, String nv) {
    if (date == null) return nv; return displayShort(date);
  }

  public static String displayShort(java.util.Date date) {
    if (date == null) return null;
    SimpleDateFormat df = new SimpleDateFormat(shortDate);
    df.setTimeZone(TimeZone.getDefault());
    return df.format(date);
  }

  public static String displayLong(java.util.Date date, String nv) {
    if (date == null) return nv; return displayLong(date);
  }
    
  public static String displayLong(java.util.Date date) {
    if (date == null) return null;
    SimpleDateFormat df = new SimpleDateFormat(longDate);
    df.setTimeZone(TimeZone.getDefault());
    return df.format(date);
  }
    
  public String displayShrunk(String nv) {
    return displayShrunk(this.date, nv);
  }

  public String displayShort(String nv) {
    return displayShort(this.date, nv);
  }

  public String displayLong(String nv) {
    return displayLong(this.date, nv);
  }

  public String displayShrunk() {
    return displayShrunk(this.date);
  }

  public String displayShort() {
    return displayShort(this.date);
  }

  public String displayLong() {
    return displayLong(this.date);
  }

}
