package org.erif.util;

import java.util.*;

import java.sql.*;

public class DbTree {

  /* extra class variables */
  protected long forestid, // unique by forests
                 treeid, // tree id unique by tree
                 id, // unique by node; == id
                 left, right, // left and right.
                 depth; // depth of node in tree -- generated on select

  /* gets and sets */
  public long getForestId() { return forestid; }
  public long getTreeId() { return treeid; }
  public long getDepth() { return depth; }
  public long getLeft() { return left; }
  public long getRight() { return right; }
  public long getId() { return id; }
  public long getChildrenCount() { return (right-left-1)/2; }

  /* methods with meat on them :) */

  /** returns all nodes "above" the given node */
  public DbTree[] getParents(Connection conn) throws SQLException {
    Vector v = new Vector();
    //PreparedStatement ps = conn.prepareStatement("SELECT f.*, count(f2.id) as depth FROM forests f, forests f2 WHERE f.forestid = ? AND f.treeid = ? AND f.lft < ? AND f.rgt > ? AND f.forestid = f2.forestid AND f.treeid = f2.treeid AND f.lft > f2.lft AND f.rgt < f2.rgt GROUP BY f.forestid, f.treeid, f.lft, f.rgt, f.id ORDER BY f.lft;");
    PreparedStatement ps = conn.prepareStatement("SELECT f.* FROM forests f WHERE f.forestid = ? AND f.treeid = ? AND f.lft < ? AND f.rgt > ? ORDER BY f.lft");
    ps.setLong(1,forestid);
    ps.setLong(2,treeid);
    ps.setLong(3,left);
    ps.setLong(4,right);
    ResultSet rs = ps.executeQuery();
    DbTree t;
    while (rs.next()) {
      t = new DbTree();
      t.init(rs);
      //t.depth=rs.getLong("depth");
      v.addElement(t);
    }
    DbTree[] forest = new DbTree[v.size()];
    v.copyInto(forest);
    return forest;
  }

  /** returns all nodes "below" the given node */
  public DbTree[] getDirectChildren(Connection conn) throws SQLException {
    Vector v = new Vector();
    PreparedStatement ps = conn.prepareStatement("SELECT f.*, count(f2.id) as depth FROM forests f, forests f2 WHERE f.forestid = ? AND f.treeid = ? AND f.lft > ? AND f.rgt < ? AND f.forestid = f2.forestid AND f.treeid = f2.treeid AND f.lft > f2.lft AND f.rgt < f2.rgt GROUP BY f.forestid, f.treeid, f.lft, f.rgt, f.id ORDER BY f.lft;");
    ps.setLong(1,forestid);
    ps.setLong(2,treeid);
    ps.setLong(3,left);
    ps.setLong(4,right);
    ResultSet rs = ps.executeQuery();
    DbTree t;
    long mindepth=-1;
    while (rs.next()) {
      t = new DbTree();
      t.init(rs);
      t.depth=rs.getLong("depth");
      v.addElement(t);
      if (mindepth < 0 || mindepth > t.depth) mindepth = t.depth;
    }
    for (int i=0;i<v.size();i++) {
      if (((DbTree)v.elementAt(i)).depth != mindepth) {
        v.removeElementAt(i);
        i--;
      }
    }
    DbTree[] forest = new DbTree[v.size()];
    v.copyInto(forest);
    return forest;
  }

  /** returns all nodes "below" the given node */
  public DbTree[] getChildren(Connection conn) throws SQLException {
    Vector v = new Vector();
    PreparedStatement ps = conn.prepareStatement("SELECT f.*, count(f2.id) as depth FROM forests f, forests f2 WHERE f.forestid = ? AND f.treeid = ? AND f.lft > ? AND f.rgt < ? AND f.forestid = f2.forestid AND f.treeid = f2.treeid AND f.lft > f2.lft AND f.rgt < f2.rgt GROUP BY f.forestid, f.treeid, f.lft, f.rgt, f.id ORDER BY f.lft;");
    ps.setLong(1,forestid);
    ps.setLong(2,treeid);
    ps.setLong(3,left);
    ps.setLong(4,right);
    ResultSet rs = ps.executeQuery();
    DbTree t;
    while (rs.next()) {
      t = new DbTree();
      t.init(rs);
      t.depth=rs.getLong("depth");
      v.addElement(t);
    }
    DbTree[] forest = new DbTree[v.size()];
    v.copyInto(forest);
    return forest;
  }

  /** removes the given node (nodes need to be unique to their tree, else ALL references will be removed when one is, and things won't update properly).  Ideally this would roll back if more than one thing was deleted. */
  public void remove(Connection conn) throws SQLException {
    //remove node from forests
    PreparedStatement ps = conn.prepareStatement("DELETE FROM forests WHERE forestid = ? AND id = ?");
    ps.setLong(1,forestid); ps.setLong(2,id);
    int lines = ps.executeUpdate();
    if (lines != 1) throw new RuntimeException("Wrong number of lines caught in tree node deletion: "+lines); // NOTDONE/TODO: roll back?

    //nodes to the left: no change

    //nodes underneath: -1l, -1r
    ps = conn.prepareStatement("UPDATE forests SET lft=lft-1, rgt=rgt-1 WHERE forestid = ? AND treeid = ? AND lft > ? AND rgt < ?"); 
    ps.setLong(1,forestid); ps.setLong(2,treeid);
    ps.setLong(3,left); ps.setLong(4,right);
    lines = ps.executeUpdate();

    //nodes above: -2r
    ps = conn.prepareStatement("UPDATE forests SET rgt=rgt-2 WHERE forestid = ? AND treeid = ? AND lft < ? AND rgt > ?");
    ps.setLong(1,forestid); ps.setLong(2,treeid);
    ps.setLong(3,left); ps.setLong(4,right);
    lines = ps.executeUpdate();

    //nodes to the right: -2l, -2r
    ps = conn.prepareStatement("UPDATE forests SET lft=lft-2, rgt=rgt-2 WHERE forestid = ? AND treeid = ? AND lft > ?"); // right
    ps.setLong(1,forestid); ps.setLong(2,treeid);
    ps.setLong(3,right);
    lines = ps.executeUpdate();
  }

  public static DbTree create(long id, Connection conn) throws SQLException { return create(0,id,conn); }
  public static DbTree create(long forestid, long id, Connection conn) throws SQLException {
    PreparedStatement ps = conn.prepareStatement("INSERT INTO forests (forestid, treeid,lft,rgt,id) VALUES (?, nextval('forests_treeid_seq'),1,2,?)");
    ps.setLong(1,forestid);
    ps.setLong(2,id);
    ps.executeUpdate();
    DbTree t = new DbTree();
    t.left=1;
    t.right=2;
    t.id=id;
    ps = conn.prepareStatement("SELECT CURRVAL('forests_treeid_seq') AS id");
    ResultSet rs = ps.executeQuery();
    rs.next();
    t.treeid = rs.getLong("id");
    return t;
  }

  public DbTree addNode(long id, Connection conn) throws SQLException { 
    PreparedStatement ps = conn.prepareStatement("UPDATE forests SET lft=lft+2 WHERE forestid = ? AND treeid = ? AND lft >= ?");
    ps.setLong(1,forestid); ps.setLong(2,treeid); ps.setLong(3,right);
    ps.executeUpdate();

    ps=conn.prepareStatement("UPDATE forests SET rgt=rgt+2 WHERE forestid = ? AND treeid = ? AND rgt >= ?");
    ps.setLong(1,forestid); ps.setLong(2,treeid); ps.setLong(3,right);
    ps.executeUpdate();

    ps=conn.prepareStatement("INSERT INTO forests (forestid,treeid,lft,rgt,id) VALUES (?,?,?,?,?)");
    ps.setLong(1,forestid); ps.setLong(2,treeid);
    ps.setLong(3,right); ps.setLong(4,right+1); ps.setLong(5,id);
    ps.executeUpdate();

    DbTree t = new DbTree();
    t.left = right;
    t.right = right+1;
    t.depth = depth+1;
    t.treeid = treeid;
    t.id = id;
    this.right+=2;
    return t;
  }

  public void init(ResultSet rs) throws SQLException {
    this.forestid = rs.getLong("forestid");
    this.treeid = rs.getLong("treeid");
    this.left = rs.getLong("lft");
    this.right = rs.getLong("rgt");
    this.id = rs.getLong("id");
    //this.depth = -1; // hmm.  can calculate. a little expensive.
  }

  /** returns treenode for the given refid for the default forest */
  public static DbTree getTree(long id, Connection conn) throws SQLException { return getTree(0,id,conn); }

  /** returns treenode for the given refid and forest */
  public static DbTree getTree(long forestid, long id, Connection conn) throws SQLException {
    PreparedStatement ps = conn.prepareStatement("SELECT * FROM forests WHERE forestid = ? AND id = ?");
    ps.setLong(1,forestid); ps.setLong(2,id);
    ResultSet rs = ps.executeQuery();
    if (!rs.next()) {
      throw new RuntimeException("No treenode found for id: "+id+" in forest: "+forestid);
    }
    DbTree t = new DbTree();
    t.init(rs);
    if (rs.next()) {
      throw new RuntimeException("More than one treenode found for id: "+id+" in forest: "+forestid);
    }
    return t;
  }

  /** returns the "top" treenode for the given refid for the default forest */
  public static DbTree getTopOfThread(long id, Connection conn) throws SQLException { return getTopOfThread(0,id,conn); }
  /** returns the "top" treenode for the given refid and forest*/
  public static DbTree getTopOfThread(long forestid, long id, Connection conn) throws SQLException {
    PreparedStatement ps = conn.prepareStatement("SELECT f2.* FROM forests f1, forests f2 where f1.forestid = ? AND f1.id = ? AND f1.forestid = f2.forestid AND f1.treeid = f2.treeid AND f2.lft = 1;");
    ps.setLong(1,forestid);
    ps.setLong(2,id);
    ResultSet rs = ps.executeQuery();
    if (!rs.next()) {
      throw new RuntimeException("No treenode \"top\" found for id: "+id+" in forest: "+forestid);
    }
    DbTree t = new DbTree();
    t.init(rs);
    if (rs.next()) {
      throw new RuntimeException("More than one treenode \"top\" found for id: "+id+" in forest: "+forestid);
    }
    return t;
}

  /** returns the thread of treeid in the default forest */
  public static DbTree[] getThread(long treeid, Connection conn) throws SQLException { return getThread(0,treeid,conn); }
  /** returns the thread of treeid in the given forest */
  public static DbTree[] getThread(long forestid, long treeid, Connection conn) throws SQLException {
    Vector v = new Vector();
    PreparedStatement ps = conn.prepareStatement("SELECT f.*, count(f2.id) as depth FROM forests f, forests f2 WHERE f.forestid = ? AND f.treeid = ? AND f.forestid = f2.forestid AND f.treeid = f2.treeid AND f.lft > f2.lft AND f.rgt < f2.rgt GROUP BY f.forestid, f.treeid, f.lft, f.rgt, f.id ORDER BY f.lft;");
    ps.setLong(1,forestid);
    ps.setLong(2,treeid);
    ResultSet rs = ps.executeQuery();
    DbTree t;
    while (rs.next()) {
      t = new DbTree();
      t.init(rs);
      t.depth=rs.getLong("depth");
      v.addElement(t);
    }
    DbTree[] forest = new DbTree[v.size()];
    v.copyInto(forest);
    return forest;
  }

  public static DbTree[] getTopLevel(Connection conn) throws SQLException { return getTopLevel(0,conn); }
  public static DbTree[] getTopLevel(long forestid, Connection conn) throws SQLException {
    Vector v = new Vector();
    PreparedStatement ps = conn.prepareStatement("SELECT f.* FROM forests f WHERE f.forestid = ? AND f.lft=1");
    ps.setLong(1,forestid);
    ResultSet rs = ps.executeQuery();
    DbTree t;
    while (rs.next()) {
      t = new DbTree();
      t.init(rs);
      v.addElement(t);
    }
    DbTree[] forest = new DbTree[v.size()];
    v.copyInto(forest);
    return forest;
  }

  /* TODO: sort list a based on the order of list b, and b's references to a... or make hashtable? or? or?*/

  public static long[] getRefIds(DbTree[] treenodes) {
    if (treenodes == null) return new long[0];
    long[] retval = new long[treenodes.length];
    for (int i=0;i<treenodes.length;i++) {
      retval[i]=treenodes[i].getId();
    }
    return retval;
  }

  /** returns a hash from refid to the actual DbTree node */
  public static Hashtable getReverseHash(DbTree[] treenodes) {
    Hashtable retval = new Hashtable();
    if (treenodes == null) return retval;
    for (int i=0;i<treenodes.length;i++) {
      retval.put(new Long(treenodes[i].getId()), treenodes[i]);
    }
    return retval;
  }

  public boolean equals(Object o) {
    return (o instanceof DbTree && id == ((DbTree)o).getId());
  }

}
