package org.erif.util;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/** mostly for conveniently parsing an exception as a String */
public class JavaUtil {

  public static byte[] readBytes(File f) throws IOException {
    FileInputStream is = new FileInputStream(f);
    byte[] buf = new byte[(int)f.length()];
    int offset = 0; int bytes=0;
    while (offset < buf.length && (bytes=is.read(buf,offset,buf.length-offset))!=-1) offset+=bytes;
    byte[] retval = new byte[offset];
    for (int i=0;i<offset;i++) retval[i]=buf[i];
    return retval;
  }

  
/*
  public static byte[] readBytes(InputStream is) throws IOException {

  }
*/

  public static String byteArrayToHex(byte[] bs) {
    StringBuffer sb = new StringBuffer();
    for (int i=0;i<bs.length;i++) sb.append(byteToHex(bs[i]));
    return sb.toString();
  }

  public static String byteToHex(byte b) {
    int high = ((b & 0xf0) >> 4);
    int low = (b & 0x0f);
    return ""+digitToHex(high)+digitToHex(low);
  }

  public static char digitToHex(int i) {
    if (i > 15) throw new IllegalArgumentException("15 is the maximum a hex digit may be.  Pbtbtbtbtbt!");
    if (i < 10) return (char)('0'+i);
    else return (char)('a'+i-10);
  }

  public static String exceptionToString(Throwable e) {
    StringWriter sw = new StringWriter();
    e.printStackTrace(new PrintWriter(sw));
    String s = sw.toString();
    //s.replaceFirst(":","\n");
    return ""+new Date()+"\n"+s.substring(0,s.indexOf(":")+1)+"\n"+s.substring(s.indexOf(":")+1,s.length());
  }

  /**
   * Creates a new Vector with the contents of the Object array.
   * @param array Objects to add to aray
   * @return New array
   */
  public static Vector arrayToVector(Object[] array) {
    Vector retval = new Vector(array.length);
    for (int i=0; i<array.length;i++)
      retval.add(array[i]);
    return retval;
  }

  /**
   * returns a hashtable that has as its keys the values of the old table.  if things are not unique it barfs.
   */
  public static Hashtable invertHashtable(Hashtable orig) {
    Object key = null;
    Object value = null;
    Hashtable inverted = new Hashtable();
    for (Enumeration e = orig.keys(); e.hasMoreElements();) {
      key = e.nextElement();
      value = orig.get(key);
      if (inverted.get(value) != null) throw new IllegalArgumentException("original hashtable did not have unique values for its keys!");
      inverted.put(value,key);
    }
    return inverted;
  }

  /**
   * builds a hash from an array of objects keying to the given method.  if array is not unique it barfs.  To get an Iterator from an Array, try Arrays.asList(Object[]).iterator().
   */
  public static Hashtable hash(Iterator array, Class c, String methodname) throws NoSuchMethodException {
    Hashtable hash = new Hashtable();
    try {
    Class[] nullargs = new Class[0];
    Object value = null;
    Object key = null;
    Method method = c.getMethod(methodname,nullargs);
    while (array.hasNext()) {
      key = array.next();
      if (hash.get(key) != null) throw new IllegalArgumentException("array was not distinct by keys!");
      value = method.invoke(key,nullargs);
      hash.put(key,value);
    }
    } catch (InvocationTargetException ite) {
    } catch (IllegalAccessException iae) {
    }
    return hash;
  }

  /**
   * builds a hash from an array of objects keying one method to the values of another.  if array is not unique it barfs.To get an Iterator from an Array, try Arrays.asList(Object[]).iterator().
   */
  public static Hashtable hash(Iterator array, Class c, String keymethodname, String valuemethodname) throws NoSuchMethodException {
    Hashtable hash = new Hashtable();
    try {
    Class[] nullargs = new Class[0];
    Object value = null;
    Object key = null;
    Object o = null;
    Method keymethod = c.getMethod(keymethodname,nullargs);
    Method valuemethod = c.getMethod(valuemethodname,nullargs);
    while (array.hasNext()) {
      o = array.next();
      key = keymethod.invoke(key,nullargs);
      if (hash.get(key) != null) throw new IllegalArgumentException("array was not distinct by keys!");
      value = valuemethod.invoke(key,nullargs);
      hash.put(key,value);
    }
    } catch (InvocationTargetException ite) {
    } catch (IllegalAccessException iae) {
    }
    return hash;
  }

  /**
   * builds a array of longs from an array of objects extracting via parameterless method.  To get an Iterator from an Array, try Arrays.asList(Object[]).iterator().
   */
  public static long[] getLongIdentifiers(Iterator array, Class c, String methodname) throws NoSuchMethodException {
    Vector v = new Vector();
    try {
    Class[] nullargs = new Class[0];
    Method method = c.getMethod(methodname,nullargs);
    while (array.hasNext()) {
      //v.addElement(new Long(method.invoke(array.next(),nullargs)));
      v.addElement((Long)method.invoke(array.next(),nullargs));
    }
    } catch (InvocationTargetException ite) {
    } catch (IllegalAccessException iae) {
    }
    long[] identifiers = new long[v.size()];
    for (int i=0;i<identifiers.length;i++) {
      identifiers[i] = ((Long)v.elementAt(i)).longValue();
    }
    return identifiers;
  }

  /** straighten an Object[][] into an Object[], essentially concatenating
   *  each following piece */
  public static Object[] straighten(Object[][] o) {
    int length = 0;
    if (o == null) return new Object[0];
    for (int i=0;i<o.length;i++) if (o[i] != null) length += o[i].length;
    Object[] straightened = new Object[length];
    for (int i=0,j=0;i<o.length;i++) {
      for (int k=0;o[i] != null && k<o[i].length;j++,k++) {
        straightened[j]=o[i][k];
      }
    }
    return straightened;
  }
  
}
