package org.erif.util;

import java.util.*;
import java.io.*;

import java.lang.reflect.*;

/** For the sorting of objects by the results of a given method
 * 
 * Usage: pass in to Array.sort().  i.e.:<br />
 *
 *<pre>
 * MyClass myarray = MyClass.getValidEntries();
 * SimpleCompare myClassFirstNameSort = new SimpleCompare(MyClass.class,"getFirstName",SimpleCompare.ASCENDING);
 * Arrays.sort(myarray,myClassFirstNameSort);
 *</pre>
 * @author Kaolin Fire Stockinger
 */

public class SimpleCompare implements Comparator {

  public static final boolean ASCENDING = true;
  public static final boolean DESCENDING = false;

  private static final Class[] nullarray = new Class[0];
  private static final Object[] nullargs = new Class[0];

  private Method method, method2;
  private boolean direction, direction2;

  public SimpleCompare(Class c, String methodname, boolean direction) throws NoSuchMethodException {
    init (c,methodname,direction);
  }

  public SimpleCompare(Class c, String methodname, boolean direction, String methodname2, boolean direction2) throws NoSuchMethodException {
    init(c,methodname,direction,methodname2,direction2);
  }

  private void init(Class c, String methodname, boolean direction) throws NoSuchMethodException {
    this.direction=direction;
    method = c.getMethod(methodname,nullarray);
/*
    returntype = method.getReturnType();
    if (!Comparable.class.isAssignableFrom(returntype))
      throw new IllegalArgumentException("returntype of method ("+returntype+") must implement Comparable!");
*/
  }

  private void init(Class c, String methodname, boolean direction, String methodname2, boolean direction2)  throws NoSuchMethodException {
    this.direction2=direction2;
    method2 = c.getMethod(methodname2,nullarray);
    init(c,methodname,direction);
  }

  public int compare(Object o1, Object o2) {
    int compare = compare(o1,o2,method,direction);
    if (compare == 0 && method2 != null) compare = compare(o1,o2,method2,direction2);
    return compare;
  }

  public int compare(Object o1, Object o2, Method m, boolean direction) {
    try {
      Object r1 = m.invoke(o1,nullargs);
      Object r2 = m.invoke(o2,nullargs);
      if (r1 == null) {
        return (r2 == null)?0:(direction?-1:1);
      } else if (r2 == null) {
        return (direction?1:-1);
      }
      if (r1 instanceof String) {
        r1 = ((String)r1).toLowerCase();
        r2 = ((String)r2).toLowerCase();
      }
      if (r1 instanceof Boolean) {
        if (r1.equals(r2)) return 0;
        if (r1.equals(Boolean.TRUE)) 
          return (direction?1:-1);
        else
          return (direction?-1:1);
      }
      if (direction == ASCENDING)
        return ((Comparable)r1).compareTo(r2);
      else
        return ((Comparable)r2).compareTo(r1);
    } catch (InvocationTargetException ite) {
    } catch (IllegalAccessException iae) {
    }
    return o1.hashCode() - o2.hashCode();
  }
	
}
